// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: intConstrained_Delaunay.c,v 1.4 2005/12/02 10:35:34 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include"stack-c.h"
#include <string.h>
#include "machine.h"
#include <math.h>
#include <stdio.h>

extern void* 		constraind_delaunay_2(double* x, double* y,double* cxy,int nbpts,int nbconstraint);
extern int* 		cdt2_get_connectivity(void* Ptr,int* NbTri);
extern double* 	cdt2_get_coord(void* Ptr,int* nbpts);
extern void 		cdt2_insert_points(void* Ptr,double* x, double* y,int nbpts);
extern void 		cdt2_remove_points(void* Ptr,double* x, double* y,int nbpts);
extern void 		cdt2_insert_constraints(void* Ptr,double* cxy,int nbconstraint);
extern void 		cdt2_remove_constraints(void* Ptr,double* cxy,int nbconstraint);
extern int 			cdt2_delete(void* Ptr);

extern int send_scilab_job(char*job);
//--------------------------------------------------------------
//			Interface Constrained_Delaunay_2
//---------------------------------------------------------------
int int_constraind_delaunay_2(char* fname)
{

	int l1, m1, n1,m2, n2, l2,m3, n3, l3,l4;
	int minlhs=1, maxlhs=2, minrhs=3, maxrhs=3;
	int n = 3,one = 1;
	int* data = 0;
	void* Ptr =0;
	int NbTri = 0;
	int j = 0;
	int i = 0;
	int k = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	GetRhsVar(2, "d", &m2, &n2, &l2);
	GetRhsVar(3, "d", &m3, &n3, &l3);// m3 correspond au nbre de contraintes. n3 = 4

	if (!(m1 == m2) ||!(n1 == n2) )
		{
			sciprint("%s: Incompatible inputs \r\n","cdelaunay");
			Error(999);
			return 0;
		}
	Ptr = constraind_delaunay_2(stk(l1), stk(l2),stk(l3),n1,m3);
	if(!Ptr){
		sciprint("%s: Incompatible inputs \r\n","cdelaunay");
		return 0;
	}
	data = cdt2_get_connectivity(Ptr,&NbTri);
	if(!data){
		sciprint("%s: Incompatible inputs \r\n","GetConnectivityOfCdt");
		return 0;
	}
	CreateVar(4, "i",&NbTri, &n, &l4);

	for(j=0; j< NbTri; j++)
	{
		*istk(l4+j)= data[3*j];
		i = j + NbTri;
		*istk(l4+i)= data[1+3*j];
		k = i + NbTri;
		*istk(l4+k)= data[2+3*j];
	}

 	free(data);
	if(Lhs == 1)cdt2_delete(Ptr);
	else CreateVarFromPtr(5, "p",&one, &one,Ptr);

	LhsVar(1) = 4;
	LhsVar(2) = 5;
	return 0;
}

int int_cdt2_get_connectivity(char* fname)
{

	int lPtr, m1, n1, l2;
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	int n = 3;
	int* data = 0;
	void* Ptr =0;
	int NbTri = 0;
	int j = 0;
	int i = 0;
	int k = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	Ptr = (void*)((unsigned long int) *stk(lPtr));
	data = cdt2_get_connectivity(Ptr,&NbTri);
	if(!data){
		sciprint("%s: Incompatible inputs \r\n","GetConnectivityOfCdt");
		sciprint("the intersections of input constraints are disallowed \r\n");
		return 0;
	}
	CreateVar(2, "i",&NbTri, &n, &l2);

	for(j=0; j< NbTri; j++)
	{
		*istk(l2+j)= data[3*j];
		i = j + NbTri;
		*istk(l2+i)= data[1+3*j];
		k = i + NbTri;
		*istk(l2+k)= data[2+3*j];
	}
	LhsVar(1) = 2;
 	free(data);
	return 0;
}
int int_cdt2_get_coord(char* fname)
{
	int lPtr,l2;
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	int n = 2,m1,n1;
	double* Coord = 0;
	int nbpts = 0;
	int j = 0;
	void* Ptr;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);

	Ptr = (void*)((unsigned long int) *stk(lPtr));

	Coord = cdt2_get_coord(Ptr,&nbpts);
	if(!Coord)return 0;

	CreateVar(2, "d",&nbpts, &n, &l2);

	for(j=0; j< 2*nbpts; j++)
	{
		*stk(l2+j)= Coord[j];
	}

	LhsVar(1) = 2;
	free(Coord);

	return 0;

}

int int_cdt2_insert_points(char* fname)
{
	int l1, m1, n1,m2, n2, m3, n3,l2, l3,lPtr;
	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);
	GetRhsVar(3, "d", &m3, &n3, &l2);

	if (!(m2 == m3) ||!(n2 == n3) )
	{
		sciprint("%s: Incompatible inputs \r\n","InsertPoints_cdt");
		Error(999);
		return 0;
	}

	if(!stk(lPtr))return 0;
	cdt2_insert_points((void*)((unsigned long int) *stk(lPtr)),stk(l1),stk(l2),n2);

	LhsVar(1) = 1;
	return 0;


}

int int_cdt2_remove_points(char* fname)
{
	int l1, m1, n1,m2, n2, m3, n3,l2, l3,lPtr;
	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);
	GetRhsVar(3, "d", &m3, &n3, &l2);

	if (!(m2 == m3) ||!(n2 == n3) )
	{
		sciprint("%s: Incompatible inputs \r\n","RemovePoints_cdt");
		Error(999);
		return 0;
	}

	if(!stk(lPtr))return 0;
	cdt2_remove_points((void*)((unsigned long int) *stk(lPtr)),stk(l1),stk(l2),n2);

	LhsVar(1) = 1;
	return 0;


}
int int_cdt2_insert_constraints(char* fname)
{
	int l1, m1, n1,m2, n2,lPtr;
	int minlhs=1, maxlhs=1, minrhs=2, maxrhs=2;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);

	if (!(n2 == 4) )
	{
		sciprint("%s: Incompatible inputs \r\n","cdt2_insert_constraints");
		Error(999);
		return 0;
	}

	if(!stk(lPtr))return 0;
	cdt2_insert_constraints((void*)((unsigned long int) *stk(lPtr)),stk(l1),m2);

	LhsVar(1) = 1;
	return 0;
}
int int_cdt2_remove_constraints(char* fname)
{
	int l1, m1, n1,m2, n2,lPtr;
	int minlhs=1, maxlhs=1, minrhs=2, maxrhs=2;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);

	if (!(n2 == 4) )
	{
		sciprint("%s: Incompatible inputs \r\n","cdt2_remove_constraints");
		Error(999);
		return 0;
	}

	if(!stk(lPtr))return 0;
	cdt2_remove_constraints((void*)((unsigned long int) *stk(lPtr)),stk(l1),m2);

	LhsVar(1) = 1;
	return 0;
}

int int_cdt2_delete(char* fname)
{
	int m,n,lPtr;
	int m2,n2,l2;
	int minrhs=0, maxrhs=2;
	int i =0;
	char* job;
	int joblength;
	CheckRhs(minrhs,maxrhs) ;

	switch(Rhs)
{
	case 1:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						if(((unsigned long int) *stk(lPtr)))
						cdt2_delete((void*)((unsigned long int) *stk(lPtr)));
				}break;
	case 2:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						GetRhsVar(2, "c", &m2, &n2, &l2);
						if(((unsigned long int) *stk(lPtr)))
						cdt2_delete((void*)((unsigned long int) *stk(lPtr)));
						joblength = 6+strlen(cstk(l2))+1;
						job = malloc(joblength);
						// clear the pointer in scilab interpreter.
						sprintf(job,"clear %s",cstk(l2));
						send_scilab_job(job);
						free(job);
					}break;
	default:{cdt2_delete(NULL);};
}
	return 0;

}

