// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: intDelaunay.c,v 1.4 2005/12/02 10:35:34 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include"stack-c.h"
#include <string.h>
#include "machine.h"
#include <math.h>
#include <stdio.h>

//extern int* Delaunay(double* x, double* y,int nbc,int* NbTri);
extern int* Delaunay_3d(double* x, double* y,double* z,int nbc,int* Nbtetra/*,double* Xf,double* Yf,double* Zf */);
extern int* Delaunay_nd(double* pts,int n,int nbpts,int* Nbsimplex,int* curdim);
extern int* Constrainedelaunay(double* x, double* y,double* cxy,int nbpts,int nbconstraint,int* NbTri);
//--------------------------------------------------------------
//			Interface Delaunay 2d
//---------------------------------------------------------------
// int intDelaunay(char* fname)
// {
//
// 	static int l1, m1, n1,m2, n2, l2, l3;
// 	static int minlhs=1, maxlhs=1, minrhs=2, maxrhs=2;
// 	static int n = 3;
// 	int* data = 0;
// 	int NbTri = 0;
// 	int j = 0;
// 	int i = 0;
// 	int k = 0;
//
// 	CheckRhs(minrhs,maxrhs) ;
// 	CheckLhs(minlhs,maxlhs) ;
//
// 	GetRhsVar(1, "d", &m1, &n1, &l1);
// 	GetRhsVar(2, "d", &m2, &n2, &l2);
//
// 	if (!(m1 == m2) ||!(n1 == n2) )
// 		{
// 			sciprint("%s: Incompatible inputs \r\n");
// 			Error(999);
// 			return 0;
// 		}
//
// 	data = Delaunay(stk(l1),stk(l2),n1,&NbTri);
// 	CreateVar(3, "i",&NbTri, &n, &l3);
//
// 	for(j=0; j< NbTri; j++)
// 	{
// 		*istk(l3+j)= data[3*j];
// 		i = j + NbTri;
// 		*istk(l3+i)= data[1+3*j];
// 		k = i + NbTri;
// 		*istk(l3+k)= data[2+3*j];
// 	}
// 	LhsVar(1) = 3;
// 	free(data);
// 	return 0;
// }
//--------------------------------------------------------------
//			Interface Delaunay_3d
//---------------------------------------------------------------
int intDelaunay_3d(char* fname)
{

	static int l1, m1, n1,m2, n2, l2,m3,n3, l3,l4,l5,l6,l7;
	static int minlhs=1, maxlhs=4, minrhs=3, maxrhs=3;
	static int n = 4;
	int* data = 0;
	int i = 0;
	int j = 0;
	int k = 0;
	int l = 0;
	int nbtetra = 0,nbfaces = 0;
	double* Xf ;
	double* Yf ;
	double* Zf ;
	int mXf = 3, nXf = 0, mYf = 3, nYf = 0, mZf = 3, nZf = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	GetRhsVar(2, "d", &m2, &n2, &l2);
	GetRhsVar(3, "d", &m3, &n3, &l3);

	if (!(m1 == m2) ||!(n1 == n2)||!(n1 == n3) )
	{
		sciprint("%s: Incompatible inputs \r\n");
		Error(999);
		return 0;
	}
	data = Delaunay_3d(stk(l1),stk(l2),stk(l3),n1,&nbtetra /*,Xf,Yf,Zf */);
	if(!data) return 0;

	CreateVar(4, "i", &nbtetra, &n, &l4);

	for(j=0; j< nbtetra; j++)
	{
		*istk(l4+j)= data[4*j];
		i = j + nbtetra;
		*istk(l4+i)= data[1+4*j];
		k = i + nbtetra;
		*istk(l4+k)= data[2+4*j];
		l = k + nbtetra;
		*istk(l4+l)= data[3+4*j];
	}
	/*
	//------------------------------------
	nXf = 4*nbtetra; nYf = 4*nbtetra; nZf = 4*nbtetra;

 	CreateVar(5, "d",&mXf, &nXf, &l5);
 	CreateVar(6, "d",&mYf, &nYf, &l6);
 	CreateVar(7, "d",&mZf, &nZf, &l7);
	for(j=0; j< 12*nbtetra; j++)
	{
		*stk(l4+j)= Xf[j];
		*stk(l5+j)= Yf[j];
		*stk(l6+j)= Zf[j];
	}
	LhsVar(2) = 5;
	LhsVar(3) = 6;
	LhsVar(4) = 7;
	free(Xf);
	free(Yf);
	free(Zf);
	*/
	free(data);
	//------------------------------------
	LhsVar(1) = 4;


	return 0;
}

//--------------------------------------------------------------
//			Interface Delaunay_d
//---------------------------------------------------------------
int intDelaunay_nd(char* fname)
{

	static int l1, m1, n1,m2, n2, l2,m3,n3, l3,l4;
	static int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	int* data = 0;
	int i = 0;
	int j = 0;
	int nbsimplices = 0;
	int curdim = 0;
	int n = 0;
	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	//GetRhsVar(2, "d", &m2, &n2, &l2);
	//GetRhsVar(3, "d", &m3, &n3, &l3);
/*
	if (!(m1 == m2) ||!(n1 == n2)||!(n1 == n3) )
	{
		sciprint("%s: Incompatible inputs \r\n",);
		Error(999);
		return 0;
	}
*/
	data = Delaunay_nd(stk(l1),n1,m1,&nbsimplices,&curdim);
	if(!data) return 0;

	// la matrice a retourner est de dimension (nbsimplices,curdim +1).

	n = curdim +1;

	CreateVar(2, "i", &nbsimplices, &n, &l2);

	for(j=0; j< nbsimplices; j++)
	{
		for(i =0; i < n; i++ )
		{
			*istk(l2+j+i*nbsimplices)= data[i+n*j];
		}
	}

	LhsVar(1) = 2;
	free(data);

	return 0;
}

//--------------------------------------------------------------
//			Interface Constrained_Delaunay_2
//---------------------------------------------------------------
int intConstrained_Delaunay(char* fname)
{

	static int l1, m1, n1,m2, n2, l2,m3, n3, l3,l4;
	static int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;
	static int n = 3,in1 = 0;
	int* data = 0;
	int NbTri = 0;
	int j = 0;
	int i = 0;
	int k = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	GetRhsVar(2, "d", &m2, &n2, &l2);
	GetRhsVar(3, "d", &m3, &n3, &l3);// m3 correspond au nbre de contraintes. n3 = 4

	if (!(m1 == m2) ||!(n1 == n2) )
		{
			sciprint("%s: Incompatible inputs \r\n");
			Error(999);
			return 0;
		}
	data = Constrainedelaunay(stk(l1), stk(l2),stk(l3),n1,m3,&NbTri);
	if(!data){
		sciprint("%s: Incompatible inputs \r\n");
		sciprint("%s: the intersections of input constraints are disallowed \r\n");
		return 0;
	}
	CreateVar(4, "i",&NbTri, &n, &l4);

	for(j=0; j< NbTri; j++)
	{
		*istk(l4+j)= data[3*j];
		i = j + NbTri;
		*istk(l4+i)= data[1+3*j];
		k = i + NbTri;
		*istk(l4+k)= data[2+3*j];
	}
	LhsVar(1) = 4;

 	free(data);
	return 0;
}



