// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: intDelaunay_2d.c,v 1.4 2005/12/02 10:35:34 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include"stack-c.h"

extern void* delaunay_triangulation_2(double* x, double* y,int nbc);
extern int* dt2_get_connectivity(void* Ptr,int*NbTri);
extern void dt2_insert_points(void* Ptr,double* x, double* y,int nbc);
extern void dt2_remove_points(void* Ptr,double* x, double* y,int nbc);
extern int dt2_delete(void* Ptr);
extern int send_scilab_job(char *job);


//--------------------------------------------------------------
//			Interface Delaunay 2d
//---------------------------------------------------------------
int int_delaunay_triangulation_2(char* fname)
{
	static int l1, m1, n1,m2, n2, l2, l3;
	static int minlhs=1, maxlhs=2, minrhs=2, maxrhs=2;
	void* Ptr;
	int one =1;
	static int n = 3;
	int* data = 0;
	int NbTri = 0;
	int j = 0;
	int i = 0;
	int k = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	GetRhsVar(2, "d", &m2, &n2, &l2);

	if (!(m1 == m2) ||!(n1 == n2) )
	{
		sciprint("%s: Incompatible inputs \r\n","delaunay");
		Error(999);
		return 0;
	}

	Ptr = delaunay_triangulation_2(stk(l1),stk(l2),n1);
	if(!Ptr)return 0;
	CreateVarFromPtr(3,"p",&one,&one,Ptr);

	data =dt2_get_connectivity(Ptr,&NbTri);
	if(!data)return 0;
	CreateVar(4, "i",&NbTri, &n, &l3);

	if(Lhs != 2)dt2_delete(Ptr);

	for(j=0; j< NbTri; j++)
	{
		*istk(l3+j)= data[3*j];
		i = j + NbTri;
		*istk(l3+i)= data[1+3*j];
		k = i + NbTri;
		*istk(l3+k)= data[2+3*j];
	}
	LhsVar(1) = 4;
	LhsVar(2) = 3;
	free(data);
	return 0;

}

int int_dt2_get_connectivity(char* fname)
{
	static int lPtr,l3;
	static int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	static int n = 3,m1,n1;
	int* data = 0;
	int NbTri = 0;
	int j = 0;
	int i = 0;
	int k = 0;
	void* Ptr;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);

	Ptr = (void*)((unsigned long int) *stk(lPtr));

	data = dt2_get_connectivity(Ptr,&NbTri);
	if(!data)return 0;

	CreateVar(2, "i",&NbTri, &n, &l3);

	for(j=0; j< NbTri; j++)
	{
		*istk(l3+j)= data[3*j];
		i = j + NbTri;
		*istk(l3+i)= data[1+3*j];
		k = i + NbTri;
		*istk(l3+k)= data[2+3*j];
	}
	LhsVar(1) = 2;
	free(data);
	return 0;

}

int int_dt2_insert_points(char* fname)
{
	static int l1, m1, n1,m2, n2, m3, n3,l2, l3,lPtr;
	static int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);
	GetRhsVar(3, "d", &m3, &n3, &l2);

	if (!(m2 == m3) ||!(n2 == n3) )
	{
		sciprint("%s: Incompatible inputs \r\n","InsertPoints");
		Error(999);
		return 0;
	}

	if(!stk(lPtr))return 0;
	dt2_insert_points((void*)((unsigned long int) *stk(lPtr)),stk(l1),stk(l2),n2);

	LhsVar(1) = 1;
	return 0;


}

int int_dt2_remove_points(char* fname)
{
	static int l1, m1, n1,m2, n2, m3, n3,l2, l3,lPtr;
	static int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);
	GetRhsVar(3, "d", &m3, &n3, &l2);

	if (!(m2 == m3) ||!(n2 == n3) )
	{
		sciprint("%s: Incompatible inputs \r\n","RemovePoints");
		Error(999);
		return 0;
	}

	if(!stk(lPtr))return 0;
	dt2_remove_points((void*)((unsigned long int) *stk(lPtr)),stk(l1),stk(l2),n2);

	LhsVar(1) = 1;
	return 0;


}
int int_dt2_delete(char* fname)
{
	int m,n,lPtr;
	int m2,n2,l2;
	int minrhs=0, maxrhs=2;
	int i =0;
	char* job;
	int joblength;
	CheckRhs(minrhs,maxrhs) ;
	switch(Rhs)
{
	case 1:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						if(((unsigned long int) *stk(lPtr)))
						dt2_delete((void*)((unsigned long int) *stk(lPtr)));
				}break;
	case 2:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						GetRhsVar(2, "c", &m2, &n2, &l2);
						if(((unsigned long int) *stk(lPtr)))
						dt2_delete((void*)((unsigned long int) *stk(lPtr)));
						joblength = 6+strlen(cstk(l2))+1;
						job = malloc(joblength);
						// clear the pointer in scilab interpreter.
						sprintf(job,"clear %s",cstk(l2));
						send_scilab_job(job);
						free(job);
					}break;
	default:{dt2_delete(NULL);};
}
	return 0;

}

