// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: intDelaunay_3d.c,v 1.5 2005/12/02 10:35:34 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include"stack-c.h"
#include <string.h>
//#include "machine.h"
//#include <math.h>
#include <stdio.h>


extern void* delaunay_triangulation_3(double* x, double* y,double* z,int nbpts);
extern int* dt3_get_connectivity(void* Ptr,int* Nbtetra);
extern void dt3_insert_points(void* Ptr,double* x, double* y,double* z,int nbpts);
extern void dt3_remove_points(void* Ptr,double* x, double* y,double* z,int nbpts);
extern int dt3_delete(void* Ptr);
extern int send_scilab_job(char *job);
//--------------------------------------------------------------
//			Interface delaunay_triangulation_3
//---------------------------------------------------------------
int int_delaunay_triangulation_3(char* fname)
{

	int l1, m1, n1,m2, n2, l2,m3,n3, l3,l4;
	int minlhs=1, maxlhs=2, minrhs=3, maxrhs=3;

	void* Ptr=0;
	int one =1;

	int* data = 0;
	int n = 4;
	int i = 0;
	int j = 0;
	int k = 0;
	int l = 0;
	int nbtetra = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	GetRhsVar(2, "d", &m2, &n2, &l2);
	GetRhsVar(3, "d", &m3, &n3, &l3);

	if (!(m1 == m2) ||!(n1 == n2)||!(n1 == n3) )
	{
		sciprint("%s: Incompatible inputs \r\n","delaunay3");
		Error(999);
		return 0;
	}
	//DeleteDelaunay((void*)((unsigned long int) *stk(lPtr)));
	Ptr = delaunay_triangulation_3(stk(l1),stk(l2),stk(l3),n1);
	if(!Ptr) return 0;

	data = dt3_get_connectivity(Ptr,&nbtetra);
	if(!data) return 0;

	CreateVar(4, "i", &nbtetra, &n, &l4);

	for(j=0; j< nbtetra; j++)
	{
		*istk(l4+j)= data[4*j];
		i = j + nbtetra;
		*istk(l4+i)= data[1+4*j];
		k = i + nbtetra;
		*istk(l4+k)= data[2+4*j];
		l = k + nbtetra;
		*istk(l4+l)= data[3+4*j];
	}
	free(data);

	if(Lhs == 1) dt3_delete(Ptr);
	else CreateVarFromPtr(5, "p", &one, &one, Ptr);
	//------------------------------------
	LhsVar(1) = 4;
	LhsVar(2) = 5;
	return 0;
}

int int_dt3_get_connectivity(char* fname)
{

	int lPtr, m1, n1, l2;
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	int n = 4;
	int* data = 0;
	int i = 0;
	int j = 0;
	int k = 0;
	int l = 0;
	int nbtetra = 0;
	void* Ptr;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	Ptr = (void*)((unsigned long int) *stk(lPtr));

	data = dt3_get_connectivity(Ptr,&nbtetra);
	if(!data) return 0;

	CreateVar(2, "i", &nbtetra, &n, &l2);

	for(j=0; j< nbtetra; j++)
	{
		*istk(l2+j)= data[4*j];
		i = j + nbtetra;
		*istk(l2+i)= data[1+4*j];
		k = i + nbtetra;
		*istk(l2+k)= data[2+4*j];
		l = k + nbtetra;
		*istk(l2+l)= data[3+4*j];
	}
	free(data);
	//------------------------------------
	LhsVar(1) = 2;

	return 0;
}

int int_dt3_insert_points(char* fname)
{
	int l1, m1, n1,m2, n2, m3, n3,m4,n4,l2, l3,lPtr;
	int minlhs=1, maxlhs=1, minrhs=4, maxrhs=4;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);
	GetRhsVar(3, "d", &m3, &n3, &l2);
	GetRhsVar(4, "d", &m4, &n4, &l3);
	if (!(m2 == m3) ||!(n2 == n3)||!(n2 == n4)||!(m3 ==m4))
	{
		sciprint("%s: Incompatible inputs \r\n","dt3_insert_points");
		Error(999);
		return 0;
	}
	if(!stk(lPtr))return 0;

	dt3_insert_points((void*)((unsigned long int) *stk(lPtr)),stk(l1),stk(l2),stk(l3),n2);

	LhsVar(1) = 1;
	return 0;


}

int int_dt3_remove_points(char* fname)
{
	int l1, m1, n1,m2, n2, m3, n3,m4,n4,l2, l3,lPtr;
  int minlhs=1, maxlhs=1, minrhs=4, maxrhs=4;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;


	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);
	GetRhsVar(3, "d", &m3, &n3, &l2);
	GetRhsVar(4, "d", &m4, &n4, &l3);

	if (!(m2 == m3) ||!(n2 == n3)||!(n2 == n4)||!(m3 ==m4) )
	{
		sciprint("%s: Incompatible inputs \r\n","dt3_remove_points");
		Error(999);
		return 0;
	}

	if(!stk(lPtr))return 0;
	dt3_remove_points((void*)((unsigned long int) *stk(lPtr)),stk(l1),stk(l2),stk(l3),n2);

	LhsVar(1) = 1;
	return 0;


}

int int_dt3_delete(char* fname)
{
	int m,n,lPtr;
	int m2,n2,l2;
	int minrhs=0, maxrhs=2;
	int i =0;
	char* job;
	int joblength;
	CheckRhs(minrhs,maxrhs) ;

	if(Rhs == 1 )
	{
		GetRhsVar(1, "p", &m, &n, &lPtr);
		if(((unsigned long int) *stk(lPtr)))
		dt3_delete((void*)((unsigned long int) *stk(lPtr)));

	}else if(Rhs == 2 )
	{
		GetRhsVar(1, "p", &m, &n, &lPtr);
		GetRhsVar(2, "c", &m2, &n2, &l2);

		if(((unsigned long int) *stk(lPtr)))
		dt3_delete((void*)((unsigned long int) *stk(lPtr)));

		joblength = 6+strlen(cstk(l2))+1;
		job = malloc(joblength);

		// clear the pointer in scilab interpreter.
		sprintf(job,"clear %s",cstk(l2));
		send_scilab_job(job);

		free(job);

	}else
	{dt3_delete(NULL);}

	return 0;

}


