// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: intDelaunay_mesher.c,v 1.4 2005/12/02 10:35:34 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include"stack-c.h"

extern void* mesh_2(double* Pslg,int nbconstraint);
extern double* mesh2_get_coord(void* Ptr,int* nbpts);
extern int* mesh2_get_connectivity(void* Ptr,int*NbTri);
extern void mesh2_refine(void* Ptr,double shapebound,double upperbound);
extern void mesh2_set_seeds(void* Ptr,double* x,double* y,int nbseeds);
extern int mesh2_delete(void* Ptr);

extern int send_scilab_job(char *job);
//--------------------------------------------------------------
//			Interface mesh_2
//---------------------------------------------------------------
int int_mesh_2(char* fname)
{
	int l1, m1, n1, l3,l4;
	int minlhs=1, maxlhs=3, minrhs=1, maxrhs=1;
	void* Ptr;
	int one =1;
	int n = 3,m = 2;
	int* data = 0;
	double* Coord = 0;
	int NbTri = 0,nbpts = 0;
	int j = 0;
	int i = 0;
	int k = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	//GetRhsVar(2, "d", &m2, &n2, &l2);

	if (!(n1 == 4) )
	{
		sciprint("%s: Incompatible inputs \r\n","mesher_2d");
		Error(999);
		return 0;
	}

	Ptr = mesh_2(stk(l1),m1);
	if(!Ptr)return 0;
	CreateVarFromPtr(2,"p",&one,&one,Ptr);

	Coord = mesh2_get_coord(Ptr,&nbpts);
	if(!Coord)return 0;
	CreateVar(3, "d",&nbpts, &m, &l3); //Coord sera une matrice (nbpts,2)

	for(j=0; j< 2*nbpts; j++)
	{
		*stk(l3+j)= Coord[j];
	}
	data =mesh2_get_connectivity(Ptr,&NbTri);
	if(!data)return 0;
	CreateVar(4, "i",&NbTri, &n, &l4);


	for(j=0; j< NbTri; j++)
	{
		*istk(l4+j)= data[3*j];
		i = j + NbTri;
		*istk(l4+i)= data[1+3*j];
		k = i + NbTri;
		*istk(l4+k)= data[2+3*j];
	}
// les variables du retour.
	LhsVar(1) = 3;
	LhsVar(2) = 4;
	LhsVar(3) = 2;

	free(data);
	free(Coord);

	return 0;

}

int int_mesh2_get_connectivity(char* fname)
{
  int lPtr,l3;
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	int n = 3,m1,n1;
	int* data = 0;
	int NbTri = 0;
	int j = 0;
	int i = 0;
	int k = 0;
	void* Ptr;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);

	Ptr = (void*)((unsigned long int) *stk(lPtr));

	data = mesh2_get_connectivity(Ptr,&NbTri);
	if(!data)return 0;

	CreateVar(2, "i",&NbTri, &n, &l3);

	for(j=0; j< NbTri; j++)
	{
		*istk(l3+j)= data[3*j];
		i = j + NbTri;
		*istk(l3+i)= data[1+3*j];
		k = i + NbTri;
		*istk(l3+k)= data[2+3*j];
	}

	LhsVar(1) = 2;
	free(data);
	return 0;

}
int int_mesh2_get_coord(char* fname)
{
	int lPtr,l2;
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	int n = 2,m1,n1;
	double* Coord = 0;
	int nbpts = 0;
	int j = 0;
	void* Ptr;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);

	Ptr = (void*)((unsigned long int) *stk(lPtr));

	Coord = mesh2_get_coord(Ptr,&nbpts);
	if(!Coord)return 0;

	CreateVar(2, "d",&nbpts, &n, &l2);

	for(j=0; j< 2*nbpts; j++)
	{
		*stk(l2+j)= Coord[j];
	}

	LhsVar(1) = 2;
	free(Coord);

	return 0;

}

int int_mesh2_refine(char* fname)
{
	int l1, m1, n1,m2, n2, m3, n3,l2,lPtr;
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=3;
	double shapebound =0.,upperbound=0.;
	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	switch(Rhs)
	{
		case 1:{
							GetRhsVar(1, "p", &m1, &n1, &lPtr);

					}break;
		case 3:{
							GetRhsVar(1, "p", &m1, &n1, &lPtr);
							GetRhsVar(2, "d", &m2, &n2, &l1);
							GetRhsVar(3, "d", &m3, &n3, &l2);
							if (!(m2 == m3) ||!(n2 == n3) ||!(n2 == 1))
							{
								sciprint("%s: Incompatible inputs \r\n","mesh2_refine");
								Error(999);
								return 0;
							}
							shapebound = *stk(l1);
							upperbound = *stk(l2);
						}break;
		default:{
							sciprint("%s: Incompatible inputs \r\n","mesh2_refine");
							Error(999);
							return 0;
						};
	}

	if(!stk(lPtr))return 0;
	mesh2_refine((void*)((unsigned long int) *stk(lPtr)),shapebound,upperbound);

	LhsVar(1) = 1;
	return 0;


}
int int_mesh2_set_seeds(char* fname)
{
	int l1, m1, n1,m2, n2, m3, n3,l2,lPtr;
	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;
	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l1);
	GetRhsVar(3, "d", &m3, &n3, &l2);

	if (!(m2 == m3) ||!(n2 == n3))
	{
		sciprint("%s: Incompatible inputs \r\n","mesh2_set_seeds");
		Error(999);
		return 0;
	}


	if(!stk(lPtr))return 0;
	mesh2_set_seeds((void*)((unsigned long int) *stk(lPtr)),stk(l1),stk(l2),n2);

	LhsVar(1) = 1;
	return 0;

}

int int_mesh2_delete(char* fname)
{
	int m,n,lPtr;
	int m2,n2,l2;
	int minrhs=0, maxrhs=2;
	int i =0;
	char* job;
	int joblength;
	CheckRhs(minrhs,maxrhs) ;
	switch(Rhs)
{
	case 1:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						if(((unsigned long int) *stk(lPtr)))
						mesh2_delete((void*)((unsigned long int) *stk(lPtr)));
				}break;
	case 2:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						GetRhsVar(2, "c", &m2, &n2, &l2);
						if(((unsigned long int) *stk(lPtr)))
						mesh2_delete((void*)((unsigned long int) *stk(lPtr)));
						joblength = 6+strlen(cstk(l2))+1;
						job = malloc(joblength);
						// clear the pointer in scilab interpreter.
						sprintf(job,"clear %s",cstk(l2));
						send_scilab_job(job);
						free(job);
					}break;
	default:{mesh2_delete(NULL);};
}
	return 0;

}

