// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: intDelaunay_nd.c,v 1.5 2005/12/02 10:35:34 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include"stack-c.h"
#include <string.h>
#include "machine.h"
#include <math.h>
#include <stdio.h>

extern void* delaunay_triangulation_n(double* pts,int n,int nbpts);
extern int* dtn_get_connectivity(void* Ptr,int* Nbsimplex,int* curdim);
extern void dtn_insert_points(void* Ptr,double* pts,int n,int nbpts);
extern int dtn_delete(void* Ptr);
//--------------------------------------------------------------
//			Interface Delaunay_d
//---------------------------------------------------------------
int int_delaunay_triangulation_n(char* fname)
{

	int l1,l2, m1, n1;
	int minlhs=1, maxlhs=2, minrhs=1, maxrhs=1;
	void* Ptr = 0;
	int one =1;

	int* data = 0;
	int i = 0;
	int j = 0;
	int nbsimplices = 0;
	int curdim = 0;
	int n = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	Ptr = delaunay_triangulation_n(stk(l1),n1,m1);
	if(!Ptr) return 0;
	data = dtn_get_connectivity(Ptr,&nbsimplices,&curdim);
	if(!data) return 0;

	// la matrice a retourner est de dimension (nbsimplices,curdim +1).

	n = curdim +1;

	CreateVar(2, "i", &nbsimplices, &n, &l2);

	for(j=0; j< nbsimplices; j++)
	{
		for(i =0; i < n; i++ )
		{
			*istk(l2+j+i*nbsimplices)= data[i+n*j];
		}
	}
	free(data);

	if(Lhs == 1)dtn_delete(Ptr);
	else CreateVarFromPtr(3, "p", &one, &one, Ptr);

	LhsVar(1) = 2;
	LhsVar(2) = 3;

	return 0;
}

int int_dtn_get_connectivity(char* fname)
{

	int lPtr, m1, n1, l2;
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	int* data = 0;
	void* Ptr = 0;
	int i = 0;
	int j = 0;
	int nbsimplices = 0;
	int curdim = 0;
	int n = 0;
	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	Ptr = (void*)((unsigned long int) *stk(lPtr));
	data = dtn_get_connectivity(Ptr,&nbsimplices,&curdim);
	if(!data) return 0;

	// la matrice a retourner est de dimension (nbsimplices,curdim +1).

	n = curdim +1;

	CreateVar(2, "i", &nbsimplices, &n, &l2);

	for(j=0; j< nbsimplices; j++)
	{
		for(i =0; i < n; i++ )
		{
			*istk(l2+j+i*nbsimplices)= data[i+n*j];
		}
	}

	LhsVar(1) = 2;
	free(data);

	return 0;
}

int int_dtn_insert_points(char* fname)
{
	static int l2, m1, n1,m2, n2,lPtr;
	static int minlhs=1, maxlhs=1, minrhs=2, maxrhs=2;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "p", &m1, &n1, &lPtr);
	GetRhsVar(2, "d", &m2, &n2, &l2);

	if(!stk(lPtr))return 0;
	dtn_insert_points((void*)((unsigned long int) *stk(lPtr)),stk(l2),n2,m2);

	LhsVar(1) = 1;
	return 0;


}

int int_dtn_delete(char* fname)
{
	int m,n,lPtr;
	int m2,n2,l2;
	int minrhs=0, maxrhs=2;
	int i =0;
	char* job;
	int joblength;
	CheckRhs(minrhs,maxrhs) ;
	switch(Rhs)
{
	case 1:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						if(((unsigned long int) *stk(lPtr)))
						dtn_delete((void*)((unsigned long int) *stk(lPtr)));
				}break;
	case 2:{
						GetRhsVar(1, "p", &m, &n, &lPtr);
						GetRhsVar(2, "c", &m2, &n2, &l2);
						if(((unsigned long int) *stk(lPtr)))
						dtn_delete((void*)((unsigned long int) *stk(lPtr)));
						joblength = 6+strlen(cstk(l2))+1;
						job = malloc(joblength);
						// clear the pointer in scilab interpreter.
						sprintf(job,"clear %s",cstk(l2));
						send_scilab_job(job);
						free(job);
					}break;
	default:{dtn_delete(NULL);};
}
	return 0;

}

