// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: Delaunay_triangulation.cpp,v 1.4 2005/12/02 10:35:35 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Cartesian.h>
#include <CGAL/Triangulation_vertex_base_with_info_3.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/Delaunay_triangulation_3.h>
#include <CGAL/Constrained_Delaunay_triangulation_2.h>
#include <CGAL/Delaunay_d.h>
#include <CGAL/Cartesian_d.h>
#include <CGAL/Tetrahedron_3.h>


//-------------------------
#include <CGAL/IO/Triangulation_geomview_ostream_3.h>
//---------------------------
#include <map>
#include <vector>
#include <iostream>
//-----------------------------------------------------------------------
//            LES ALORITHMES DE TRIANGULATION DELAUNAY 1D, 2D, 3D ET nD
/********************************************************************/
//					Delaunay 2D
//
/********************************************************************/
// typedef CGAL::Cartesian<double>  								K;
// typedef CGAL::Delaunay_triangulation_2<K>  						Delaunay2d;
// typedef Delaunay2d::Point   									Point;
// typedef Delaunay2d::Vertex_handle 								Vertex_handle;
// typedef Delaunay2d::Finite_vertices_iterator  					Vertices_iterator;
// typedef Delaunay2d::Finite_faces_iterator 						faces_iterator;
// typedef Delaunay2d::Edge  	  									Edge;
// typedef Delaunay2d::Finite_edges_iterator       				Edges_iterator;
//
//
// extern "C"
// {
//     int* Delaunay(double* x, double* y,int nbc,int* NbTri)
// 	{
// 	// nbc nember of points.
// 	// TRI is (nbc,3) array of triangles, each row contains one triangle.
//  		int* TRI = 0;
// 		Delaunay2d dt;
//
// 		std::map<Vertex_handle,int> listVertex;
// 		std::vector<int> IndTRI;
//
// 		int ind = 1;
// 		for(int i=0; i< nbc; i++)
// 		{
// 			Point P(x[i],y[i]);
// 			Vertex_handle vertex = dt.insert(P);
// 			listVertex[vertex] = ind;
// 			ind ++;
// 		}
//
// 		int count = 0;
// 		for(faces_iterator iter = dt.finite_faces_begin(); iter!= dt.finite_faces_end();++iter)
// 		{
// 			Vertex_handle vertex1 = (*iter).vertex(0);
// 			Vertex_handle vertex2 = (*iter).vertex(1);
// 			Vertex_handle vertex3 = (*iter).vertex(2);
//
// 			IndTRI.push_back(listVertex[vertex1]);//printf("%i  ",listVertex[vertex1]);
// 			IndTRI.push_back(listVertex[vertex2]);//printf("%i  ",listVertex[vertex2]);
// 			IndTRI.push_back(listVertex[vertex3]);//printf("%i  \n",listVertex[vertex3]);
//
// 			count++;
// 		}
//
// 		unsigned int nbtri = IndTRI.size();
// 		TRI = (int*)malloc(nbtri*sizeof(int));
//
// 		if(!TRI)return 0;
//
// 		for(unsigned int j=0; j< nbtri; j++)
// 		{
// 			TRI[j] = IndTRI[j];
// 		}
//
// 		*NbTri = count;
// 		return TRI;
// 	}
//
// }
/********************************************************************/
//					Delaunay 3D
//
/********************************************************************/
// typedef CGAL::Exact_predicates_inexact_constructions_kernel 				Kernel;
// typedef CGAL::Triangulation_vertex_base_with_info_3<int, Kernel>  			Vb;
// typedef CGAL::Triangulation_data_structure_3<Vb>                     		Tds;
// typedef CGAL::Delaunay_triangulation_3<Kernel, Tds>                       	Delaunay3d;
// typedef Delaunay3d::Point                          							Point_3;
// typedef Delaunay3d::Cell_handle                    							Cell_handle;
// typedef Delaunay3d::Vertex_handle    										vertex_handle;
// typedef Delaunay3d::Finite_cells_iterator 									cells_iterator;
// typedef CGAL::Tetrahedron_3<Kernel>											Tetrahedron;
//
// extern "C"
// {
//    	int* Delaunay_3d(double* x, double* y,double* z,int nbc,int* Nbtetra)
// 	{
// 	nbc nember of coordinates.
// 	TES is (nbc,4) matrix of tetra, each row contains one Tetrahedron.
// 		int* TES = 0;
// 		Delaunay3d dt;
//
// 		std::map<vertex_handle,int> listVertex;
// 		std::vector<int> IndTRI;
//
// 		int ind = 1;
// 		for(int i=0; i< nbc; i++)
// 		{
// 			Point_3 P(x[i],y[i],z[i]);
// 			vertex_handle vertex = dt.insert(P);
// 			listVertex[vertex] = ind;
// 			vertex->info() = ind;
// 			ind ++;
// 		}
// 		int count = 0;
// 		for(cells_iterator iter = dt.finite_cells_begin(); iter!= dt.finite_cells_end();++iter)
// 		{
// 			vertex_handle vertex1 = (*iter).vertex(0);
// 			vertex_handle vertex2 = (*iter).vertex(1);
// 			vertex_handle vertex3 = (*iter).vertex(2);
// 			vertex_handle vertex4 = (*iter).vertex(3);
//
// 			IndTRI.push_back(listVertex[vertex1]);//printf("%i  ",listVertex[vertex1]);
// 			IndTRI.push_back(listVertex[vertex2]);//printf("%i  ",listVertex[vertex2]);
// 			IndTRI.push_back(listVertex[vertex3]);//printf("%i  ",listVertex[vertex3]);
// 			IndTRI.push_back(listVertex[vertex4]);//printf("%i \n",listVertex[vertex4]);
// 			count++;
//
// 		}
//
// 		*Nbtetra = count;
//
// 		unsigned int nbInd = IndTRI.size();
// 		TES = (int*)malloc(nbInd*sizeof(int));
//
// 		if(!TES) return 0;
//
// 		for(unsigned int j=0; j< nbInd; j++)
// 		{
// 			TES[j] = IndTRI[j];
// 		}
// 		return TES;
// 	}
// }
/********************************************************************/
//					Delaunay nD
//
/********************************************************************/
typedef CGAL::Cartesian_d<double>  			Ker;
typedef CGAL::Delaunay_d<Ker> 				Delaunay_d;
typedef Delaunay_d::Point_d                Point_d;
typedef Delaunay_d::Simplex_iterator 		Simplex_iterator;
typedef Delaunay_d::Vertex_handle 			Vertex_handle_d;

extern"C"
{
	//--------------------------------------------------------------------------
	//	Inputs:
	// 			nbpts	: 	# of points.
	//			n 		:	space dimension.
	//			curdim	: 	current dimension, the affine dimension.
	//			pts 	:	(nbpts,n) array representing nbpts points of n-D space
	//	Output:
	// 			TES 	:	(Nbsimplex,curdim+1) array of simplices, each row contains one simplex.
	//--------------------------------------------------------------------------

   	int* Delaunay_nd(double* pts,int n,int nbpts,int* Nbsimplex,int* curdim)
	{
		int* TES = 0	;
		Delaunay_d dt(n);

		std::map<Vertex_handle_d,int> listVertex;
		std::vector<int> IndTRI;

		int ind = 1;
		for(int j=0; j< nbpts; j++)
		{

			double*  coord = new double[n];
			for(int i=0; i< n; i++)
			{
				coord[i]= pts[i*nbpts+j];
			}
			Point_d P(n,coord,coord+n);
			Vertex_handle_d vertex = dt.insert(P);
			delete[] coord;
			listVertex[vertex] = ind;
			ind ++;
		}
		// on recupere la dimemson du l'espace courant.
		int dcur = dt.current_dimension();
			*curdim = dcur;
		int count = 0;
		for(Simplex_iterator iter = dt.simplices_begin(); iter!= dt.simplices_end();++iter)
		{
			for(int i =0; i <= dcur; i++ )
			{
				Vertex_handle_d vertex = dt.vertex_of_simplex(iter,i);
				IndTRI.push_back(listVertex[vertex]);//printf("%i  ",listVertex[vertex]);
			}
			//
			count++;
		}
		// on recupere le nombre de simplexes.
		*Nbsimplex = count;

		unsigned int nbInd = IndTRI.size();
		TES = (int*)malloc(nbInd*sizeof(int));

		if(!TES) return 0;

		for(unsigned int j=0; j< nbInd; j++)
		{
			TES[j] = IndTRI[j];
		}
		return TES;
	}
}

//-------------------------
/********************************************************************/
//					Constrained_Delaunay_triangulation_2
//
/********************************************************************/
struct Kern : CGAL::Exact_predicates_inexact_constructions_kernel 		{};
typedef CGAL::Triangulation_vertex_base_2<Kern>                     	cVb;
typedef CGAL::Constrained_triangulation_face_base_2<Kern>          		Fb;
typedef CGAL::Triangulation_data_structure_2<cVb,Fb>              		TDS;
//typedef CGAL::Exact_predicates_tag                               		Itag;
typedef CGAL::Constrained_Delaunay_triangulation_2<Kern, TDS/*, Itag*/> 	CDT;
typedef CDT::Point         										 		Point_2;
typedef CDT::Vertex_handle 												cVertex_handle;
typedef CDT::Finite_faces_iterator 										cfaces_iterator;

extern "C"
{
    int* Constrainedelaunay(double* x, double* y,double* cxy,int nbpts,int nbconstraint,int* NbTri)
	{
	// nbpts nember of points.
	// nbconstraint nember of Constrained edges.
	// cxy (nbconstraint,4) array of constrained segments.
	// TRI is (nbc,3) array of triangles, each row contains one triangle.

 		int* TRI = 0;
		CDT cdt;

		std::map<cVertex_handle,int> listVertex;
		std::vector<int> IndTRI;

		for(int i=0; i< nbpts; i++)
		{
			Point_2 P(x[i],y[i]);
			cdt.insert(P);
		}
		// insertion des contraintes.
		for(int i=0; i< nbconstraint; i++)
		{
			Point_2 P1(cxy[i],cxy[i+nbconstraint]);
			Point_2 P2(cxy[i+2*nbconstraint],cxy[i+3*nbconstraint]);
			//cVertex_handle vertex1 = cdt.insert(P1);
			//cVertex_handle vertex2 = cdt.insert(P2);

			cdt.insert_constraint(P1,P2);
		}

		if(cdt.is_valid())
		{
			CDT::Finite_vertices_iterator vit;
			int ind = 1;
			for (vit = cdt.finite_vertices_begin(); vit != cdt.finite_vertices_end(); ++vit)
			{
				listVertex[vit] = ind;
				ind ++;
			}
			int count = 0;
			for(cfaces_iterator iter = cdt.finite_faces_begin(); iter!= cdt.finite_faces_end();++iter)
			{
				cVertex_handle vertex1 = (*iter).vertex(0);
				cVertex_handle vertex2 = (*iter).vertex(1);
				cVertex_handle vertex3 = (*iter).vertex(2);

				IndTRI.push_back(listVertex[vertex1]);
				IndTRI.push_back(listVertex[vertex2]);
				IndTRI.push_back(listVertex[vertex3]);

				count++;
			}
			*NbTri = count;
// 			if(nbpts != cdt.number_of_vertices())
// 			{
// 				// pour l'instant l'algo ne supporte que les contraintes sans intersection.
// 				return 0;
// 			}else
// 			{
				unsigned int nbtri = IndTRI.size();
				TRI = (int*)malloc(nbtri*sizeof(int));

				if(!TRI)return 0;

				for(unsigned int j=0; j< nbtri; j++)
				{
					TRI[j] = IndTRI[j];
				}
				return TRI;
			//}
		}else return 0;
	}


}





